/*
 *  $Id: networkmonitorinterface.h,v 1.2 2001/11/02 12:38:20 dreibh Exp $
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 1999-2001 by Thomas Dreibholz
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at www.sctp.de which should be used for
 * any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          dreibh@exp-math.uni-essen.de
 *
 * Purpose: Network Monitor Interface
 *
 */


#ifndef NETWORKMONITORINTERFACE_H
#define NETWORKMONITORINTERFACE_H


#include "tdsystem.h"
#include "networkmonitorreport.h"
#include "synchronizable.h"


namespace Coral {


/**
  * This class is an interface for a network monitor, which sums bytes and
  * packets transmitted.
  *
  * @short   Network Monitor Interface
  * @author  Thomas Dreibholz (dreibh@exp-math.uni-essen.de)
  * @version 1.0
  */
class NetworkMonitorInterface
{
   // ====== Virtual destructor =============================================
   public:
   /**
     * Virtual destructor.
     */
   virtual ~NetworkMonitorInterface() = 0;


   // ====== Status functions ===============================================
   /**
     * Check, if NetworkMonitor is ready.
     *
     * @return true, if ready; false, if not.
     */
   virtual bool ready() const = 0;


   // ====== Lock functions =================================================
   /**
     * Lock monitor to access data.
     */
   virtual void lock() = 0;

   /**
     * Unlock monitor.
     */
   virtual void unlock() = 0;


   // ====== Report functions ===============================================
   /**
     * Reset the network report.
     */
   virtual void resetReport() = 0;

   /**
     * Get a copy of the network report created by the monitor.
     *
     * @param report Address of NetworkMonitorReport to store the report.
     */
   virtual void getReport(NetworkMonitorReport* report) = 0;

   /**
     * Get a copy of the network report created by the monitor, then reset
     * the report. Note: Do *not* use getReport() and resetReport() to
     * get a report and reset the monitor's report. The monitor could have
     * received packets between getReport() and resetReport()!
     *
     * @param report Address of NetworkMonitorReport to store the report.
     */   
   virtual void moveReport(NetworkMonitorReport* report) = 0;
};


}


#endif
