/*
 *  $Id: streamreport.cc,v 1.2 2001/11/02 12:38:20 dreibh Exp $
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 1999-2001 by Thomas Dreibholz
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at www.sctp.de which should be used for
 * any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          dreibh@exp-math.uni-essen.de
 *
 * Purpose: Stream Monitor Report
 *
 */


#include "tdsystem.h"
#include "streamreport.h"
#include "tools.h"


namespace Coral {


// ###### Constructor #######################################################
StreamReport::StreamReport()
{
}


// ###### Reset #############################################################
void StreamReport::reset()
{
   Source.reset();
   Destination.reset();
   FlowLabel = 0;
   Protocol3 = 0x00;
   Protocol4 = 0x00;
   StreamID  = 0;
   pad[0] = pad[1] = 0x00;
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      BytesRaw[i]     = 0;
      BytesPayload[i] = 0;
      Packets[i]      = 0;
   }
}


// ###### Translate #########################################################
void StreamReport::translate()
{
   FlowLabel = translate32(FlowLabel);
   StreamID  = translate32(StreamID);
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      BytesRaw[i]     = translate64(BytesRaw[i]);
      BytesPayload[i] = translate64(BytesPayload[i]);
      Packets[i]      = translate32(Packets[i]);
   }
}


// ###### == operator #######################################################
int StreamReport::operator==(const StreamReport& report) const
{
   return((Source      == report.Source)      &&
          (Destination == report.Destination) &&
          (FlowLabel   == report.FlowLabel)   &&
          (StreamID    == report.StreamID)    &&
          (Protocol3   == report.Protocol3)   &&
          (Protocol4   == report.Protocol4));
}


// ###### != operator #######################################################
int StreamReport::operator!=(const StreamReport& report) const
{
   return((Source      != report.Source)      ||
          (Destination != report.Destination) ||
          (FlowLabel   != report.FlowLabel)   ||
          (StreamID    != report.StreamID)    ||
          (Protocol3   != report.Protocol3)   ||
          (Protocol4   != report.Protocol4));
}


// ###### Output operator ###################################################
ostream& operator<<(ostream& os, const StreamReport& report)
{
   os << InternetAddress(report.Source) << " -> " << InternetAddress(report.Destination);
   char str[64];
   if(report.FlowLabel != 0) {
      snprintf((char*)&str,sizeof(str),", Flowlabel $%05x",report.FlowLabel);
      os << str;
   }
   snprintf((char*)&str,sizeof(str),", Protocols $%02x/$%02x, SID %d",
            report.Protocol4,report.Protocol3,report.StreamID);
   os << str << endl;

   os << "   Raw     = ";
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      if(report.BytesRaw[i] != 0) {
         os << report.BytesRaw[i] << "(" << TrafficClassValues::getNameForIndex(i) << ") ";
      }
   }
   os << endl;
   os << "   Payload = ";
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      if(report.BytesPayload[i] != 0) {
         os << report.BytesPayload[i] << "(" << TrafficClassValues::getNameForIndex(i) << ") ";
      }
   }
   os << endl;
   os << "   Packets = ";
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      if(report.Packets[i] != 0) {
         os << report.Packets[i] << "(" << TrafficClassValues::getNameForIndex(i) << ") ";
      }
   }
   os << endl;
   return(os);
}


}
