/*
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Purpose: configuration parser and storage
 * Author : Sim Woon Chiat Siemens ICM N PG U SE D2
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "configuration.h"
#include "glib.h"


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if defined (LINUX)
 #include <string.h>
 #include <unistd.h>
#endif




configParam configList[MAX_NO_CONFIG_PARAMS];


/* Private utility functions */ 
void initConfigList() {

  int i;

  for (i=0;i<MAX_NO_CONFIG_PARAMS;i++) {
    strcpy(configList[i].key,"");
    strcpy(configList[i].value,"");
  }

}


int insertKeyValue(char *key, char *value) {

  int i;

  /* Find an empty slot */
  for (i=0;i<MAX_NO_CONFIG_PARAMS;i++) {
    if (strcmp(configList[i].key,"")==0) {
      strcpy(configList[i].key,key);
      strcpy(configList[i].value,value);
      return 1;
    }
  }

  return 0;
}


/* Function used to separate a configuration string into its key and value */
int separateKeyValue(char *str, char *key, char *value) {

  /* Not the optimum */

  int i,j,k;

  i=j=k=0;

 
  /* search for equal sign */
  for (i=0;i<strlen(str);i++) {

    if (str[i]=='=') {

      /* Copy the key */      
      strncpy(key,str,i);

      /* Copy the value minus the \n */
      str = str + (i+1);
      strncpy(value,str,strlen(str)-1);

      return 1;
    }

  }

  return 0;

}



/* Public Interface functions */
/* Read the configuration file specified and populate the internal hash table
   with the key-value pairs */
int readConfig(unsigned char *configFileName) {

  FILE *configFile;
  char configLine[255];
  char key[50];
  char value[200];

  /* Initialization the list */
  initConfigList();

  /* Open configuration file */
  if ((configFile = fopen(configFileName,"r"))!=NULL) {

    /* Loop through the lines in the file */
    while (fgets(configLine,255,configFile)) {

      bzero (key,sizeof(key));
      bzero (value,sizeof(value));

      /* Separate the key-value pair */
      if (separateKeyValue(configLine,key,value)) {
	
	/* Insert into the internal hash table */
	insertKeyValue(key,value);

      }

    }

    fclose(configFile);

    return 1;

  }							 
  

  return 0; 

}



/* Search for the value for a specific key */
/* Should return an array of params with same key */
int  findValue(char *key,configParam params[MAX_NO_CONFIG_PARAMS],int *noFound) {

  int i;

  *noFound=0;

  for (i=0;i<MAX_NO_CONFIG_PARAMS;i++) {
    if (strcmp(configList[i].key,key)==0) {
      params[(*noFound)++] = configList[i];
    }
  }

  if (noFound) return 1;

  return 0;
}



/* Debug function */
void printConfig() {

  int i;

  for (i=0;i<MAX_NO_CONFIG_PARAMS;i++) {
    if (strcmp(configList[i].key,"")!=0) {
      fprintf(stdout,"Key = %s, Value = %s\n",configList[i].key,configList[i].value);
    }
  }


}

