/*
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Purpose: Module contains functions to write logfiles (functionally 
 * not complete)
 * Author : Sim Woon Chiat Siemens ICM N PG U SE D2
 *
 */

#include <stdio.h>
#include <time.h>
#include <errno.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if defined (LINUX)
 #include <unistd.h>
#endif

#include "logger.h"

FILE *logFilePtr;
short logFileOpened=0;
short logLevel;

int startLogger(char *logFile,short l) {

  char *timeAsString;
  time_t now;


  if ((logFilePtr=fopen(logFile,"w"))!=NULL) {
   
    /* Successful in opening log file */

    logFileOpened = 1;
    logLevel = l;

    /* Write the current date and time*/

    time(&now);
    timeAsString = (char*)ctime(&now);  
    fprintf(logFilePtr,"Log started on : %s\n",timeAsString);
    fflush(logFilePtr);

    return 1;
  }

  else {
    fprintf(stdout,"Unable to open log file, %s,\nError = %s\n",logFile,strerror(errno));
    fflush(stdout);
  }
  
  return 0;

}

void logMsg(char *msg,short msgType) {

  char m[255];
  char t[255];

  char *timeAsString;
  time_t now;

  bzero(m,sizeof(t));
  bzero(m,sizeof(m));

  /* Generate the header for different msgType */
  switch(msgType) {
  case monStatus:
    sprintf(t,"[Daemon Status] : ");
    break;

  case monSCTPD:
    sprintf(t,"[SCTPd] : ");
    break;

  case monUA:
    sprintf(t,"[UA] : ");
    break;

  case monSCTPStatus:
    sprintf(t,"[SCTP status] : ");
    break;

  case monTesting:
    sprintf(t,"[Testing] : ");
    break;

  case monRaw:
    sprintf(t," ");
    break;
  default:
    sprintf(t,"[Unclassified] : ");
    break;

  }

  /* Append the user message to the header */
  strcat(t,msg);


  /* Determine the verbosity of the message and add accordingly */
  switch (logLevel) {    
  case LOG_LEVEL_1:

    
    break;

  case LOG_LEVEL_3:

    
    break;

  case LOG_LEVEL_5:

    
    break;

  case LOG_LEVEL_2: case LOG_LEVEL_4: case LOG_LEVEL_6:

    /* Get the timestamp */
    time(&now);
    timeAsString = (char*)ctime(&now);  
    strncat(m,timeAsString,strlen(timeAsString)-1);
    strcat(m," ");
    break;
  }

  strcat(m,t);

  switch (logLevel) {

    /* Print to log file */
  case LOG_LEVEL_3: case LOG_LEVEL_4:
    if ((strcmp(m,"")!=0) && logFileOpened) {

      fprintf(logFilePtr,m);
      fflush(logFilePtr);
    }

    break;

    /* print to screen */
  case LOG_LEVEL_1:case LOG_LEVEL_2:

    if (strcmp(m,"")!=0) {

      fprintf(stdout,m);
      fflush(stdout);
    }

    break;

  
    /* Print to both screen and logfile */
  case LOG_LEVEL_5: case LOG_LEVEL_6:
    if (strcmp(m,"")!=0) {

      if (logFileOpened) {
	fprintf(logFilePtr,m);
	fflush(logFilePtr);
      }
      fprintf(stdout,m);
      fflush(stdout);
    }


  }
}


int stopLogger() {


  return fclose(logFilePtr);

}

