/*
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Purpose: Module (header) for handling heartbeats between daemon and UAs 
 * Author : Sim Woon Chiat Siemens ICM N PG U SE D2
 *
 */


#ifndef PACEMAKER_H
#define PACEMAKER_H

#include <time.h>



/* Data structure to store information about heartbeats sent out */
typedef 
struct heart_beat {

  unsigned int heartbeat_id;
  time_t time_sent;
  unsigned short ua_id;
  unsigned int timer_id;

} Heartbeat;

/* Heartbeat Data structs List Management Functions -------------------------*/

/* Insert a heartbeat into State and call sctp_startTimer func of the 
   SCTP library.
   ua_id : ID of UA that this heartbeat is for
   timer_id : The ID of the timer running for this heartbeat
   RETURN : id of heartbeat if success, 0 failure
*/
int insertHeartbeat(unsigned short ua_id,unsigned int timer_id);

/* Remove a specific timer, stop it as well 
   heartbeat_id : ID of heartbeat to remove
   RETURN : 1 success, 0 failure
*/
int removeHeartbeat(unsigned int heartbeat_id);

/* Find a particular Heartbeat 
   heartbeat_id : ID of heartbeat to search for
   heartbeat : Pointer to data structure found.
   RETURN : 1 success, 0 failure
*/
int findHeartbeat (unsigned int heartbeat_id,Heartbeat **heartbeat);

int findHeartbeatForUA(unsigned short ua_id);


/* ----------------------------------------------------------------------- */


/* Callback function to handle the case when a 
   HEARTBEAT_ACK message arrives */
void handle_heartbeat_ack (unsigned int heartbeat_id) ;

/* Callback function to handle the case when a 
   HEARTBEAT message expired */
void handle_heartbeat_expired (unsigned int timer_id);

/* Callback function to handle the case when the interval
   between HEARTBEAT messages expired */
void handle_heartbeat_int_expired (unsigned int timer_id);

/* Just a wrapper around the function provided by sctpdutils */
int send_Heartbeat_To_UA(unsigned short ua_id,unsigned int timer_id);

/* Start the Pacemaker for a particular UA */
int startPacemaker (unsigned short ua_id);

/* Cancel Timer for a Particular heartbeat */
int cancelTimersForUA(unsigned short ua_id);

/* Set the heartbeat interval */
void setHeartbeatInt(unsigned int t);

/* Set the heartbeat timeout */
void setHeartbeatTimeout(unsigned int t);

#endif





