/*
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Purpose: utility functions for the SCTP demon program
 * Author : Sim Woon Chiat Siemens ICM N PG U SE D2
 *
 */

#include <stdio.h>
#include "sctpd_state.h"
#include "sctpdutils.h"
#include "../sctpdlib/sctpudputils.h"


/* Utility function */

void sendMesgToUAAssocID(int assocID,char *buf) {

  int port;

  if ((port = getUDPNotifPort(assocID))!=0) {
    sendnRawData(port,buf,MAX_UDP_DATA_LENGTH);
  }
  else {
    fprintf(stdout,"Unable to obtain the UDP notification port of the UA holding assocID=%d\n",
	    assocID);
    fflush(stdout);
  }
}


void sendMesgToUA(unsigned short ua_id,char *buf) {

  int port;

  if ((port = getUDPNotifPortForUA(ua_id))!=-1) {
    sendnRawData(port,buf,MAX_UDP_DATA_LENGTH);
  }
  else {
    fprintf(stdout,
	    "Unable to obtain the UDP notification port of the UA holding UA_ID=%d\nProbably dead\n",
	    ua_id);
    fflush(stdout);
  }
}



void sendRespToUAAssocID(int assocID,char *buf) {

  int port;

  if ((port = getUDPRespPort(assocID))!=-1) {
    sendnRawData(port,buf,MAX_UDP_DATA_LENGTH);
  }
  else {
    fprintf(stdout,"Unable to obtain the UDP notification port of the UA holding assocID=%d\n",
	    assocID);
    fflush(stdout);
  }
}


void sendRespToUA(unsigned short ua_id,char *buf){

  int port;

  if ((port = getUDPRespPortForUA(ua_id))!=-1) {
    sendnRawData(port,buf,MAX_UDP_DATA_LENGTH);
  }
  else {
    fprintf(stdout,"Unable to obtain the UDP notification port of the UA holding UA_ID=%d\n",
	    ua_id);
    fflush(stdout);
  }

}



void sendnMesgToUAAssocID(int assocID,char *buf,int length) {

  int port;

  if ((port = getUDPNotifPort(assocID))!=0) {
    sendnRawData(port,buf,length);
  }
  else {
    fprintf(stdout,"Unable to obtain the UDP notification port of the UA holding assocID=%d\n",
	    assocID);
    fflush(stdout);
  }

}

void sendnMesgToUA(unsigned short ua_id,char *buf,int length) {

  int port;

  if ((port = getUDPNotifPortForUA(ua_id))!=-1) {
    sendnRawData(port,buf,length);
  }
  else {
    fprintf(stdout,
	    "Unable to obtain the UDP notification port of the UA holding UA_ID=%d\nProbably dead\n",
	    ua_id);
    fflush(stdout);
  }

}

void sendnRespToUAAssocID(int assocID,char *buf,int length) {

  int port;

  if ((port = getUDPRespPort(assocID))>0) {
    sendnRawData(port,buf,length);
  }
  else {
    fprintf(stdout,"Unable to obtain the UDP notification port of the UA holding assocID=%d\n",
	    assocID);
    fflush(stdout);
  }

}


void sendnRespToUA(unsigned short ua_id,char *buf,int length) {

  int port;

  if ((port = getUDPRespPortForUA(ua_id))>0) {
    sendnRawData(port,buf,length);
  }
  else {
    fprintf(stdout,"Unable to obtain the UDP notification port of the UA holding UA_ID=%d\n",
	    ua_id);
    fflush(stdout);
  }
}





/* Functions used by SCTPd */

int notify_UA_communicationUp (int assocID,short status,int noOfPaths , 
			       short noOfInStreams, short noOfOutStreams,void* ulpDataPtr) {

  fnCall fc;
  unsigned char udpdataarr[MAX_UDP_DATA_LENGTH];
  int len=0;

  /* create a UDP mesg for the notification */
  create_func_call (&fc,CB_COMMUNICATION_UP_NOTIF);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(assocID),&assocID);
  add_func_arg(&fc,ARG_TYPE_SHORT,sizeof(status),&status);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(noOfPaths),&noOfPaths);
  add_func_arg(&fc,ARG_TYPE_SHORT,sizeof(noOfInStreams),&noOfInStreams);
  add_func_arg(&fc,ARG_TYPE_SHORT,sizeof(noOfOutStreams),&noOfOutStreams);
  add_func_arg(&fc,ARG_TYPE_VOID_PTR,sizeof(ulpDataPtr),&ulpDataPtr);

  /* pack and send the notifcation */
  len=pack_function_call(&fc,udpdataarr);
  sendnMesgToUAAssocID(assocID,udpdataarr,len);

  printf("CommunicationUp Notification Sent\n");

  return 0;
}


int notify_UA_shutdownComplete(int assocID,unsigned short ua_id,void* ulpDataPtr) {

  fnCall fc;
  unsigned char udpdataarr[MAX_UDP_DATA_LENGTH];
  int len=0;

  /* Create the notification message */
  create_func_call (&fc,CB_SHUTDOWN_COMPLETE_NOTIF);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(assocID),&assocID);
  add_func_arg(&fc,ARG_TYPE_VOID_PTR,sizeof(ulpDataPtr),&ulpDataPtr);

  /* pack and send the notifcation */
  len=pack_function_call(&fc,udpdataarr);
  sendnMesgToUA(ua_id,udpdataarr,len);

  printf("ShutdownComplete Notification Sent\n");

  return 0;

}


int notify_UA_networkStatusChange (int assocID,short affectedPathID , short newState,
				   void* ulpDataPtr) {

  fnCall fc;
  unsigned char udpdataarr[MAX_UDP_DATA_LENGTH];
  int len=0;

  /* Create the notification message */
  create_func_call (&fc,CB_NETWORK_STATUS_CHANGE_NOTIF);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(assocID),&assocID);
  add_func_arg(&fc,ARG_TYPE_SHORT,sizeof(affectedPathID),&affectedPathID);
  add_func_arg(&fc,ARG_TYPE_SHORT,sizeof(newState),&newState);
  add_func_arg(&fc,ARG_TYPE_VOID_PTR,sizeof(ulpDataPtr),&ulpDataPtr);

  /* pack and send the notifcation */
  len=pack_function_call(&fc,udpdataarr);
  sendnMesgToUAAssocID(assocID,udpdataarr,len);

  printf("NetworkStatusChange Notification Sent\n");

  return 0;

}


int notify_UA_dataArrive (int assocID,int streamID,int length,int protoID,
			  int unordered,void* ulpDataPtr) {


  fnCall fc;
  unsigned char udpdataarr[MAX_UDP_DATA_LENGTH];
  int len=0;

  create_func_call (&fc,CB_DATA_ARRIVE_NOTIF);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(assocID),&assocID);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(streamID),&streamID);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(length),&length);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(protoID),&protoID);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(unordered),&unordered);
  add_func_arg(&fc,ARG_TYPE_VOID_PTR,sizeof(ulpDataPtr),&ulpDataPtr);

  /* pack and send the notifcation */
  len=pack_function_call(&fc,udpdataarr);
  sendnMesgToUAAssocID(assocID,udpdataarr,len);

  printf("DataArrive Notification Sent\n");

  return 0;
}

int notify_UA_communicationLost (unsigned int assocID, unsigned short ua_id, 
				 unsigned short status,void* ulpDataPtr) {

  fnCall fc;
  unsigned char udpdataarr[MAX_UDP_DATA_LENGTH];
  int len=0;

  create_func_call (&fc,CB_COMMUNICATION_LOST_NOTIF);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(assocID),&assocID);
  add_func_arg(&fc,ARG_TYPE_SHORT,sizeof(status),&status);
  add_func_arg(&fc,ARG_TYPE_VOID_PTR,sizeof(ulpDataPtr),&ulpDataPtr);

  /* pack and send the notifcation */
  len=pack_function_call(&fc,udpdataarr);
  sendnMesgToUA(ua_id,udpdataarr,len);

  printf("CommunicationLost Notification Sent\n");

  return 0;

}



int notify_UA_restart(unsigned int assocID, void* ulpDataPtr) {

  fnCall fc;
  unsigned char udpdataarr[MAX_UDP_DATA_LENGTH];
  int len=0;

  create_func_call (&fc,CB_RESTART_NOTIF);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(assocID),&assocID);
  add_func_arg(&fc,ARG_TYPE_VOID_PTR,sizeof(ulpDataPtr),&ulpDataPtr);
  
  /* pack and send the notifcation */
  len=pack_function_call(&fc,udpdataarr);
  sendnMesgToUAAssocID(assocID,udpdataarr,len);

  printf("Restart Notification Sent\n");

  return 0;

}


int notify_UA_sendFailure(unsigned int assocID,
			  unsigned char *unsent_data, unsigned int dataLength, 
			  unsigned int *context, void* dummy) {


  fnCall fc;
  unsigned char udpdataarr[MAX_UDP_DATA_LENGTH];
  int len=0;

  create_func_call (&fc,CB_SEND_FAILURE_NOTIF);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(assocID),&assocID);
  add_func_arg(&fc,ARG_TYPE_CHAR_N,dataLength,unsent_data);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(dataLength),&dataLength);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(*context),context);
  add_func_arg(&fc,ARG_TYPE_VOID_PTR,sizeof(dummy),&dummy);  

  /* pack and send the notifcation */
  len=pack_function_call(&fc,udpdataarr);
  sendnMesgToUAAssocID(assocID,udpdataarr,len);

  printf("Sent Failure Notification Sent\n");

  return 0;

}


int notify_UA_communicationError(unsigned int assocID, unsigned short status, void* dummy) {

  fnCall fc;
  unsigned char udpdataarr[MAX_UDP_DATA_LENGTH];
  int len=0;

  create_func_call (&fc,CB_COMMUNICATION_ERROR_NOTIF);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(assocID),&assocID);
  add_func_arg(&fc,ARG_TYPE_SHORT,sizeof(status),&status);
  add_func_arg(&fc,ARG_TYPE_VOID_PTR,sizeof(dummy),&dummy);  

  /* pack and send the notifcation */
  len=pack_function_call(&fc,udpdataarr);
  sendnMesgToUAAssocID(assocID,udpdataarr,len);

  printf("CommunicationError Notification Sent\n");

  return 0;

}


int notify_queueStatusChange(unsigned int assocID, 
			     int queueType,
			     int queueID, 
			     int queueLength, 
			     void* ulpDataPtr) {
  
  fnCall fc;
  unsigned char udpdataarr[MAX_UDP_DATA_LENGTH];
  int len=0;

  create_func_call (&fc,CB_QUEUE_STATUS_CHANGE_NOTIF);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(assocID),&assocID);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(queueType),&queueType);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(queueID),&queueID);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(queueLength),&queueLength);
  add_func_arg(&fc,ARG_TYPE_VOID_PTR,sizeof(ulpDataPtr),&ulpDataPtr);  

  /* pack and send the notifcation */
  len=pack_function_call(&fc,udpdataarr);
  sendnMesgToUAAssocID(assocID,udpdataarr,len);

  printf("QueueStatusChange Notification Sent\n");

  return 0;

}


int notify_asconfStatus (unsigned int assocID, 
			 unsigned int correlationID, 
			 int result, 
			 void* tempstruct, 
			 void* ulpDataPtr) {

  fnCall fc;
  unsigned char udpdataarr[MAX_UDP_DATA_LENGTH];
  int len=0;

  create_func_call (&fc,CB_ASCONF_STATUS_NOTIF);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(assocID),&assocID);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(correlationID),&correlationID);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(result),&result);
  add_func_arg(&fc,ARG_TYPE_VOID_PTR,sizeof(tempstruct),&tempstruct);  
  add_func_arg(&fc,ARG_TYPE_VOID_PTR,sizeof(ulpDataPtr),&ulpDataPtr);  

  /* pack and send the notifcation */
  len=pack_function_call(&fc,udpdataarr);
  sendnMesgToUAAssocID(assocID,udpdataarr,len);

  return 0;


}


/* Send HEARTBEAT message to specified UA 
   ua_id : ID of UA to send heartbeat to
   RETURN : 0 = failure, 1 = success 
*/
int send_heartbeat(unsigned short ua_id,unsigned int heartbeat_id) {

  fnCall fc;
  unsigned char udpdataarr[MAX_UDP_DATA_LENGTH];
  int len=0;

  create_func_call (&fc,UDP_HEARTBEAT);
  add_func_arg(&fc,ARG_TYPE_INT,sizeof(heartbeat_id),&heartbeat_id);

  /* pack and send the notifcation */
  len=pack_function_call(&fc,udpdataarr);
  sendnMesgToUA(ua_id,udpdataarr,len);

#ifdef SHOW_HEARTBEATS
  fprintf(stdout,"Heartbeat Sent\n");
  fflush(stdout);
#endif

  return 1;

}

















