/*
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Purpose: header for utility functions for the SCTP demon program
 * Author : Sim Woon Chiat Siemens ICM N PG U SE D2
 */

#include "sctp.h"

#ifndef SCTPDUTILS_H
#define SCTPDUTILS_H


/* Functions used by SCTPd */

int notify_UA_communicationUp (int assocID,short status,int noOfPaths , 
			       short noOfInStreams, short noOfOutStreams, void* ulpDataPtr);
int notify_UA_shutdownComplete (int assocID, unsigned short ua_id,void* ulpDataPtr);
int notify_UA_networkStatusChange (int assocID,short affectedPathID , short newState,
				   void* ulpDataPtr);
int notify_UA_dataArrive (int assocID,int streamID,int length,int protoID,int unordered, 
			  void* ulpDataPtr);
int notify_UA_communicationLost (unsigned int assocID, unsigned short ua_id,
				 unsigned short status, void* ulpDataPtr);
int notify_UA_restart(unsigned int assocID, void* ulpDataPtr);
int notify_UA_sendFailure(unsigned int assocID,
			  unsigned char *unsent_data, unsigned int dataLength, 
			  unsigned int *context, void* ulpDataPtr);
int notify_UA_communicationError(unsigned int assocID, unsigned short status, void* ulpDataPtr);

/**
 * indicates that a queue length has exceeded (or length has dropped
 * below) a previously determined limit
 *  @param 0 associationID
 *  @param 1 queue type (in-queue, out-queue, stream queue etc.)
 *  @param 2 queue identifier (maybe for streams ? 0 if not used)
 *  @param 3 queue length (either bytes or messages - depending on type)
 *  @param 4 pointer to ULP data
 */
int notify_queueStatusChange (unsigned int assocID, 
			      int queueType,
			      int queueID, 
			      int queueLength, 
			      void* ulpDataPtr);

/**
 * indicates that a ASCONF request from the ULP has succeeded or failed.
 *  @param 0 associationID
 *  @param 1 correlation ID
 *  @param 2 result (int, negative for error)
 *  @param 3 pointer to a temporary, request specific structure (NULL if not needed)
 *  @param 4 pointer to ULP data
 */
int notify_asconfStatus (unsigned int assocID, 
			 unsigned int correlationID, 
			 int result, 
			 void* tempstruct, 
			 void* ulpDataPtr);

int send_heartbeat(unsigned short ua_id,unsigned int heartbeat_id);

void sendMesgToUAAssocID(int assocID,char *buf);
void sendMesgToUA(unsigned short ua_id,char *buf);
void sendRespToUAAssocID(int assocID,char *buf);
void sendRespToUA(unsigned short ua_id,char *buf);

void sendnMesgToUAAssocID(int assocID,char *buf,int length);
void sendnMesgToUA(unsigned short ua_id,char *buf,int length);
void sendnRespToUAAssocID(int assocID,char *buf,int length);
void sendnRespToUA(unsigned short ua_id,char *buf,int length);

#endif
