/*
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Daemon-based daytime server with GUI
 * Author : Sim Woon Chiat Siemens ICM N PG U SE D2
 *
 */


#include <stdio.h>
#include <stdlib.h>         /* for atoi() under Linux */

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>

#include "../sctpdlib/ua.h"

#include <glib.h>
#include <gtk/gtk.h>

#include "fontchooser.h"

/* DEFINITION OF CONSTANTS ------------------------------------------ */
#define DEFAULT_X_SIZE    500
#define DEFAULT_Y_SIZE    500

#define UDP_Notif_Port      5234
#define UDP_Resp_Port   5235
#define SCTP_App_Port     13                  /* Daytime service port */
#define MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES 5

pid_t echo_tool_pid;

/* NETWORK VARIABLES ------------------------------------------------ */
static int udpfd;                    // file descriptor to the UDP

/* GTK VARIABLES ---------------------------------------------------- */
GtkWidget *mainWindow;
GtkWidget *statusWindow;
GtkWidget *clock;

/* Other variables */
static unsigned char localAddressList[MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES][SCTP_MAX_IP_LEN];
static unsigned short noOfLocalAddresses = 0;
static int unknownCommand = 0;
static char tempstr[255];
static gint timer_id;


/* EXIT FUNCTIONS */
void cleanUpanddExit() {

  gtk_main_quit();

}


void CloseMonitor (GtkWidget * window, gpointer data){

  /* Stop timer */
  gtk_timeout_remove(timer_id);

  /* Stop sctpdlib */
  sctpd_stopLibrary();
 
  cleanUpanddExit();


}


void ChangeFontForClock (GtkWidget * window, gpointer data) {

  GtkStyle * style;
  GdkFont * font;

  font = ChooseFont("Choose a font for the Clock");
  if (font!=NULL) {
    style = gtk_style_copy (gtk_widget_get_style(clock));
    style->font = font;
    gtk_widget_set_style (clock,style);
  }

}



/* define functions to handle the GUI ------------------------------- */


void displayStatus(char *status) {

  static noLines =0;
  gtk_container_add(GTK_CONTAINER(statusWindow),gtk_list_item_new_with_label(status));
  gtk_widget_show_all(statusWindow);


}


/* Timer function to display the time */
gint timeout_callback(gpointer data) {

  char *timeAsString;
  time_t now;

  /* get the current time and convert to string */
  time(&now);
  timeAsString = (char*)ctime(&now);
 
  gtk_label_set_text(GTK_LABEL(clock),timeAsString);

  return 1;

}


// Code canabalize from book ------------------------------------------- 
/*
 * CloseDialog
 *
 * Close the dialog window.  The dialog handle is passed
 * in as the data.
 */
void CloseDialog (GtkWidget *widget, gpointer data) {

    /* --- Close it --- */
    gtk_widget_destroy (GTK_WIDGET (data));

}

/*
 * ClosingDialog
 *
 * Calls when window is about to close.  Returns FALSE
 * to let it close.  Releases the grab which made the dialog
 * window modal.
 */
void ClosingDialog (GtkWidget *widget, gpointer data){

    gtk_grab_remove (GTK_WIDGET (widget));
}



/*
 * Popup
 *
 * Display a popup dialog window with a message and an
 * "Ok" button
 */
void Popup (char *szMessage) {

    static GtkWidget *label;
    GtkWidget *button;
    GtkWidget *dialog_window;

    /* --- Create a dialog window --- */
    dialog_window = gtk_dialog_new ();

    /* --- Trap the window close signal to release the grab --- */
    gtk_signal_connect (GTK_OBJECT (dialog_window), "destroy",
                     GTK_SIGNAL_FUNC (ClosingDialog),
                     &dialog_window);

    /* --- Add a title to the window --- */
    gtk_window_set_title (GTK_WINDOW (dialog_window), "Popup");

    /* --- Create a small border --- */
    gtk_container_border_width (GTK_CONTAINER (dialog_window), 5);

    /* -------------------------- */
    /* --- Create the message --- */
    /* -------------------------- */

    /* --- Create the message in a label --- */
    label = gtk_label_new (szMessage);

    /* --- Put some room around the label --- */
    gtk_misc_set_padding (GTK_MISC (label), 10, 10);

    /* --- Add the label to the dialog box --- */
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->vbox),
                        label, TRUE, TRUE, 0);

    /* --- Make the label visible --- */
    gtk_widget_show (label);

    /* ------------------- */
    /* --- "Ok" button --- */
    /* ------------------- */

    /* --- Create the "Ok" button --- */
    button = gtk_button_new_with_label ("Ok");

    /* --- Need to close the window if they press "Ok" --- */
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                     GTK_SIGNAL_FUNC (CloseDialog),
                     dialog_window);

    /* --- Allow it to be the default button --- */
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

    /* --- Add the button to the dialog box --- */
    gtk_box_pack_start (GTK_BOX (
                  GTK_DIALOG (dialog_window)->action_area),
                 button, TRUE, TRUE, 0);

    /* --- Make the button the default button --- */
    gtk_widget_grab_default (button);

    /* --- Make the button visible --- */
    gtk_widget_show (button);

    /* --- Make the dialog box visible --- */
    gtk_widget_show (dialog_window);

    gtk_grab_add (dialog_window);
}

/* -- End of canabalization ---------------------------------------------- */



void menuExitHandler(GtkMenuItem * menuitem,gpointer data) {

  cleanUpanddExit();

}


/* Arguments Functions ----------------------------------------------------- */
void printUsage(void) {

  printf("Usage:   sctpdaytime_server [options] -s source_addr_1 -s source_addr_2 ...\n");

}

void getArgs(int argc, char **argv) {

    int c;
    extern char *optarg;
    extern int optind;

    while ((c = getopt(argc, argv, "s:")) != -1)
    {
        switch (c) {
        case 's':
            if ((noOfLocalAddresses < MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES) &&
                (strlen(optarg) < SCTP_MAX_IP_LEN  )) {
                strcpy(localAddressList[noOfLocalAddresses], optarg);
                noOfLocalAddresses++;
            }
            break;
        default:
            unknownCommand = 1;
            break;
        }
    }
}

void checkArgs(void) {
    int abortProgram;
    int printUsageInfo;
    
    abortProgram = 0;
    printUsageInfo = 0;
    
    if (noOfLocalAddresses == 0) {
        printf("Error:   At least one sourceaddress must be specified.\n");
        abortProgram = 1;
        printUsageInfo = 1;
    }
    if (unknownCommand ==1) {
         printf("Error:   Unkown options in command.\n");
         printUsageInfo = 1;
    }
    
    if (printUsageInfo == 1)
        printUsage();
    if (abortProgram == 1)
        exit(-1);
}



/* UDP Callback Handlers ------------------------------------------------ */
void* communicationUpNotif(unsigned int assocID, unsigned short status,
                           int noOfPaths,unsigned short noOfInStreams, 
			   unsigned short noOfOutStreams, void* dummy){

  char *timeAsString;
  char *tstr;
  time_t now;

  int no_of_args,i;
  unsigned char *p;
  SCTP_AssociationStatus assocStatus;
  SCTP_PathStatus pathStatus;

  displayStatus("Reveived a CommunicationUpNotif from SCTP layer");
  
  /* get the current time and convert to string */
  time(&now);
  timeAsString = (char*)ctime(&now);
   
  sctpd_send(assocID,
		 0,
		 timeAsString, strlen(timeAsString),
		 SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
		 SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, 
		 SCTP_INFINITE_LIFETIME, SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED);
    
  sctpd_shutdown(assocID);


}


void shutdownCompleteNotif(unsigned int assocID, void* ulpDataPtr) {

  displayStatus("Shutdown completed message received!");

}


void networkStatusChangeNotif(unsigned int assocID, short affectedPathID, 
			      unsigned short newState, void* ulpDataPtr){

  displayStatus("Network status changed Notification message received!");
  
  /* Alert user */
  Popup("Network Status Changed");

}


void communicationLostNotif(unsigned int assocID, unsigned short status, 
			    void* ulpDataPtr) {

  char tstr[255];

  sprintf(tstr,"Communication Lost for association (ID=%d)",assocID);
  displayStatus(tstr);

}



void udp_recv_callback (gpointer data,gint source,
			GdkInputCondition condition) {

  char recvbuff[256];
  socklen_t len;
  struct sockaddr_in taddr;
  ssize_t mesglen;

  mesglen = recvfrom(source,recvbuff,sizeof(recvbuff),0,(struct sockaddr *) &taddr,&len);

  if (mesglen>0) {

    /* call the dispatcher provided by sctpudputils which will
       call the registered UDP callback functions. */
    dispatchUAMesg(recvbuff);

  }

}



/* Bind to the UDP port */
int initializeUDPListening(int port) {

  int n,sockflags;
  socklen_t len;
  struct sockaddr_in uaddr;
  struct sockaddr_in paddr;
  unsigned char udparr[MAX_UDP_DATA_LENGTH];

  udpfd = socket(AF_INET, SOCK_DGRAM,0);

  bzero (&uaddr,sizeof(uaddr));
  bzero (&paddr,sizeof(paddr));

  uaddr.sin_family = AF_INET;
  uaddr.sin_port = htons(port);
  uaddr.sin_addr.s_addr=htonl(INADDR_ANY);
  
  if ((bind(udpfd, (struct sockaddr *) &uaddr, sizeof(uaddr)))==0) {

    // set socket as non-blocking
    sockflags = fcntl (udpfd,F_GETFL,0);
    fcntl (udpfd,F_SETFL, sockflags | O_NONBLOCK);

    // register the fd with the event handler of GTK
    gdk_input_add(udpfd,GDK_INPUT_READ,udp_recv_callback,NULL);

    displayStatus("Sucessfully binded to the application UDP port");

  }
  else {

    displayStatus("unable to bind to UDP port");

  }
}



void BuildInterface() {

  GtkWidget *mainHBox;
  GtkWidget *menubar;
  GtkWidget *actionMenu;
  GtkWidget *actionMenuItem;
  GtkWidget *menuitem;
  GtkWidget *statusScrollWin;  
  GtkStyle * style;
  GdkFont * font;
  
  gchar *titles[3] = {"AssocID","Destination Address","Status"};

  // create and set the parameters of the window
  mainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_position(GTK_WINDOW(mainWindow), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW(mainWindow), DEFAULT_X_SIZE,DEFAULT_Y_SIZE);
  gtk_window_set_title (GTK_WINDOW(mainWindow), "SCTP Daytime Server");

  // connect the signal handlers for the window
  gtk_signal_connect (GTK_OBJECT(mainWindow), "destroy", GTK_SIGNAL_FUNC(CloseMonitor), NULL);

  // create the main packing box
  mainHBox = gtk_vbox_new (FALSE,5);
  gtk_container_add (GTK_CONTAINER(mainWindow), mainHBox);


  /* MENU */
  menubar=gtk_menu_bar_new();
 
  actionMenuItem = gtk_menu_item_new_with_label("Action");
  gtk_menu_bar_append(GTK_MENU_BAR(menubar),actionMenuItem);

  // Action Menu
  actionMenu = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(actionMenuItem),actionMenu);

  menuitem =  gtk_menu_item_new_with_label("Change Font");
  gtk_menu_append(GTK_MENU(actionMenu),menuitem);
  gtk_signal_connect(GTK_OBJECT(menuitem),"activate",GTK_SIGNAL_FUNC(ChangeFontForClock),NULL);

  menuitem =  gtk_menu_item_new_with_label("Exit");
  gtk_menu_append(GTK_MENU(actionMenu),menuitem);
  gtk_signal_connect(GTK_OBJECT(menuitem),"activate",GTK_SIGNAL_FUNC(menuExitHandler),NULL);  
  gtk_box_pack_start (GTK_BOX(mainHBox), menubar, FALSE, FALSE, 0);


  /* Clock */
  clock = gtk_label_new("");
  font = gdk_font_load("-Adobe-Times-Medium-R-Normal--24-240-75-75-P-124-ISO8859-1");

  if (font!=NULL) {
    style = gtk_style_copy (gtk_widget_get_style(clock));
    style->font = font;
    gtk_widget_set_style (clock,style);
  }

  gtk_box_pack_start (GTK_BOX(mainHBox), clock, TRUE, TRUE, 0);

  /* Status Window */
  
  statusWindow = gtk_list_new();   
  statusScrollWin = gtk_scrolled_window_new(GTK_ADJUSTMENT(gtk_adjustment_new (0,0,0,0,0,0)),
					       GTK_ADJUSTMENT(gtk_adjustment_new (0,0,0,0,0,0)));
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(statusScrollWin),statusWindow);
  gtk_box_pack_start (GTK_BOX(mainHBox), statusScrollWin, TRUE, TRUE, 0);
 
  /* display the window on the screen */
  gtk_widget_show_all(mainWindow);  

}



gint main (gint argc, gchar * argv[]) {

  UDP_UA_CB udp_callbacks;
 
  /* handle all command line options */
  getArgs(argc, argv);
  checkArgs();

  /* Required GTK initialization call */
  gtk_init (&argc, &argv);

  /* Build the user interface */
  BuildInterface();

  /* Start clock */
  timer_id = gtk_timeout_add(1000,timeout_callback,NULL);

  /* Register the UDP callbacks with the dispatch mechanism */
  udp_callbacks.communicationUpNotif = &communicationUpNotif;
  udp_callbacks.shutdownCompleteNotif = &shutdownCompleteNotif;
  udp_callbacks.networkStatusChangeNotif = &networkStatusChangeNotif;

  /* Start listening on the UDP port on which the daemon sends notification */
  initializeUDPListening(UDP_Notif_Port);
  
  /* Initialize sctpdlib */
  sctpd_initLibrary(UDP_Notif_Port,UDP_Resp_Port);

  /* Call sctpd_registerInstance */
  sctpd_registerInstance(SCTP_App_Port,10,10,noOfLocalAddresses,
			 localAddressList,udp_callbacks);

  /* Start the GTK loop */
  gtk_main();

  return 0;

}



