/*
 *  $Id: sctpudputils.h,v 1.3 2001/11/13 18:29:38 ajung Exp $
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Purpose: header for UDP utility functions for the SCTP daemon program
 * Author : Sim Woon Chiat ICM N PG U SE D2
 */


#ifndef SCTPUDPUTILS_H
#define SCTPUDPUTILS_H

#include "sctp.h"
#include "ua.h"

#define FN_GET_LOCAL_ADDR                  3

#define FN_SCTP_ASSOCIATE                 10
#define FN_SCTP_SHUTDOWN                  20
#define FN_SCTP_ABORT                     30
#define FN_SCTP_GET_ASSOC_STATUS          40
#define FN_SCTP_GET_PATH_STATUS           50
#define FN_SCTP_SEND                      60
#define FN_SCTP_RECEIVE                   70
#define FN_SCTP_GET_ASSOC_DEFAULTS        80
#define FN_SCTP_SET_ASSOC_STATUS          90
#define FN_SCTP_SET_PATH_STATUS          100
#define FN_SCTP_SET_ASSOC_DEFAULTS       110
#define FN_SCTP_GET_PRIMARY              120
#define FN_SCTP_SET_PRIMARY              130
#define FN_SCTP_GET_SRTT_REPORT          140
#define FN_SCTP_CHANGE_HEARTBEAT         150
#define FN_SCTP_REQUEST_HEARTBEAT        160
#define FN_SCTP_SET_FAILURE_THRES        170
#define FN_SCTP_DELETE_ASSOC             180
#define FN_SCTPD_INITLIBRARY             183
#define FN_SCTPD_STOPLIBRARY             186
#define FN_SCTP_REGISTER_INSTANCE        190
#define FN_SCTP_UNREGISTER_INSTANCE      200
#define FN_SCTP_GET_INSTANCE_ID          210
#define FN_SCTP_RECEIVE_UNSENT           220
#define FN_SCTP_RECEIVE_UNACK            230
#define FN_SCTP_ADD_IP_ADDRESS           240
#define FN_SCTP_DELETE_IP_ADDRESS        250
#define FN_SCTP_SET_REMOTE_PRIMARY       260
#define FN_SCTP_GET_NO_LOCAL_ADDR        270
#define FN_SCTP_GET_LOCAL_ADDR           280


#define CB_COMMUNICATION_UP_NOTIF        300
#define CB_NETWORK_STATUS_CHANGE_NOTIF   310
#define CB_DATA_ARRIVE_NOTIF             320
#define CB_SHUTDOWN_COMPLETE_NOTIF       330
#define CB_SEND_FAILURE_NOTIF            340
#define CB_COMMUNICATION_LOST_NOTIF      350
#define CB_COMMUNICATION_ERROR_NOTIF     360
#define CB_RESTART_NOTIF                 370
#define CB_QUEUE_STATUS_CHANGE_NOTIF     380
#define CB_ASCONF_STATUS_NOTIF           390

#define RESP_SCTP_GET_ASSOC_STATUS       500
#define RESP_SCTP_GET_PATH_STATUS        510
#define RESP_SCTP_SHUTDOWN               520
#define RESP_SCTP_ABORT                  530
#define RESP_SCTP_SEND                   540
#define RESP_SCTP_RECEIVE                550
#define RESP_SCTP_GET_ASSOC_DEFAULTS     560
#define RESP_SCTP_ASSOCIATE              570
#define RESP_SCTP_SET_ASSOC_STATUS       580
#define RESP_SCTP_SET_PATH_STATUS        590
#define RESP_SCTP_SET_ASSOC_DEFAULTS     600
#define RESP_SCTP_GET_PRIMARY            610
#define RESP_SCTP_SET_PRIMARY            620
#define RESP_SCTP_GET_SRTT_REPORT        630
#define RESP_SCTP_CHANGE_HEARTBEAT       640
#define RESP_SCTP_REQUEST_HEARTBEAT      650
#define RESP_SCTP_SET_FAILURE_THRES      660
#define RESP_SCTP_DELETE_ASSOC           670
#define RESP_GET_LOCAL_ADDR              680
#define RESP_SCTPD_INITLIBRARY           690
#define RESP_SCTPD_STOPLIBRARY           700
#define RESP_SCTP_REGISTER_INSTANCE      710
#define RESP_SCTP_UNREGISTER_INSTANCE    720
#define RESP_SCTP_GET_INSTANCE_ID        730
#define RESP_SCTP_RECEIVE_UNSENT         740
#define RESP_SCTP_RECEIVE_UNACK          750
#define RESP_SCTP_ADD_IP_ADDRESS         760
#define RESP_SCTP_DELETE_IP_ADDRESS      770
#define RESP_SCTP_SET_REMOTE_PRIMARY     780
#define RESP_SCTP_GET_NO_LOCAL_ADDR      790
#define RESP_SCTP_GET_LOCAL_ADDR         800


#define UDP_HEARTBEAT                    900
#define UDP_HEARTBEAT_ACK                910
#define HEARTBEAT_EXPIRED                920
#define HEARTBEAT_INT_EXPIRED            930

#define MAX_NO_OF_ARGS              30
#define MAX_UDP_DATA_LENGTH         1024

#define SCTPD_PORT                  2359

/* define the constants representing different arguement types
   that can be passed. */

#define ARG_TYPE_SHORT              10
#define ARG_TYPE_INT                11
#define ARG_TYPE_LONG               12
#define ARG_TYPE_CHAR_N             13
#define ARG_TYPE_ASSOC_STATUS       14
#define ARG_TYPE_PATH_STATUS        15
#define ARG_TYPE_VOID_PTR           16
#define ARG_TYPE_INSTANCE_PARAMS    17
#define ARG_TYPE_ARRAY              18

#define sctpdPort                   3457

typedef 
struct function_spec {

  unsigned short call_id;
  unsigned short func_id;
  unsigned short no_of_args;

} fnSpec;


typedef
struct arg_spec {

  unsigned short arg_type;
  unsigned short arg_size;

} argSpec;


typedef 
struct function_call {

  fnSpec  fs;
  argSpec as[MAX_NO_OF_ARGS];
  void *dataptr[MAX_NO_OF_ARGS];

} fnCall;



typedef
struct UDP_SCTPD_Callbacks
{

  void (*handle_sctpd_initLibrary) (unsigned short,int ,int) ;

  void (*handle_sctpd_stopLibrary) (unsigned short);

  void (*handle_sctp_registerInstance)(int,             /* extra: ua id */
				       unsigned short,
				       unsigned short,
				       unsigned short,
				       unsigned int,
				       unsigned char [][SCTP_MAX_IP_LEN]);

  void (*handle_sctp_unregisterInstance)(unsigned short instance_name);
  
  void (*handle_sctp_associate)(unsigned short,
				unsigned short,
				unsigned char *,
				unsigned short, 
				void *);

  void (*handle_sctp_shutdown)(unsigned int);

  void (*handle_sctp_abort)(unsigned int);

  void (*handle_sctp_send)(unsigned int, 
			   unsigned short,
			   unsigned char *,
			   unsigned int, 
			   unsigned int, 
			   short,
			   void *, 
			   unsigned int,
			   int, 
			   int);

  void (*handle_sctp_receive)(unsigned int,
			      unsigned short, 
			      unsigned char *,
			      unsigned int *,
			      unsigned int );

  void (*handle_sctp_setAssocDefaults) (unsigned short, 
					SCTP_InstanceParameters *);

  void (*handle_sctp_getAssocDefaults) (unsigned short);

  void (*handle_sctp_getAssocStatus) (unsigned int );

  void (*handle_sctp_setAssocStatus) (unsigned int, 
				      SCTP_AssociationStatus *);

  void (*handle_sctp_getPathStatus) (unsigned int, 
				     short);

  void (*handle_sctp_setPathStatus) (unsigned int , 
				     short, 
				     SCTP_PathStatus *);

  void (*handle_sctp_setPrimary) (unsigned int , 
				  short );

  void (*handle_sctp_getPrimary) (unsigned int);

  void (*handle_sctp_getSrttReport) (unsigned int , short);

  void (*handle_sctp_changeHeartBeat) (unsigned int ,  
				       short , 
				       int , 
				       unsigned int );

  void (*handle_sctp_requestHeartbeat) (unsigned int , short );

  void (*handle_sctp_setFailureThreshold) (unsigned int, unsigned short );

  void (*handle_sctp_deleteAssociation)(unsigned int);

  void (*handle_sctp_getInstanceID)(unsigned int, unsigned short*);

  void (*handle_sctp_receiveUnsent)(unsigned int , unsigned char *, 
				    unsigned int *,unsigned short *, 
				    unsigned short *,unsigned int* );

  void (*handle_sctp_receiveUnacked)(unsigned int , unsigned char *, 
				     unsigned int *,unsigned short *, 
				     unsigned short *,unsigned int* );

  void (*handle_sctp_addIPAddress)(unsigned int ,
				   unsigned char address[SCTP_MAX_IP_LEN],
				   unsigned int *);

  void (*handle_sctp_deleteIPAddress)(unsigned int , 
				      unsigned char address[SCTP_MAX_IP_LEN],
				      unsigned int* );

  void (*handle_sctp_setRemotePrimary)(unsigned int , 
				       unsigned char address[SCTP_MAX_IP_LEN]);


  void (*handle_sctp_getNumberOfLocalAddresses)(unsigned int );

  void (*handle_sctp_getLocalAddress)(unsigned int , int );

  void (*handle_get_local_addr) (unsigned short);

  void (*handle_heartbeat_ack) (unsigned int);

  void (*handle_heartbeat_expired) (unsigned int);

  void (*handle_heartbeat_int_expired) (unsigned int);

}UDP_SCTPD_CB;



/* Utility functions, are used for the generating of UDP requests, responses 
   and callbacks */

/*
int register_UA_UDP_Callbacks(UDP_UA_CB * udpcb);
*/

int register_UA_UDP_Callbacks2(UDP_UA_CB udpcb);

int register_UA_UDP_Callbacks(UDP_UA_CB udpcb);

// int register_SCTPD_UDP_Callbacks(UDP_SCTPD_CB *udpcb);

int register_SCTPD_UDP_Callbacks(UDP_SCTPD_CB udpcb);

int sendnRawData(unsigned int port,
		 unsigned char mesg[MAX_UDP_DATA_LENGTH],
		 int length);

int create_func_call(fnCall *funcCall,unsigned short func_id);

int add_func_arg(fnCall *funcCall,unsigned short arg_type,unsigned short arg_size,void *arg_ptr);

int free_func_args(fnCall *funcCall);

int pack_function_call(fnCall *funcCall, char buf[MAX_UDP_DATA_LENGTH]);

int unpack_function_call(char buf[MAX_UDP_DATA_LENGTH],fnCall *funcCall);

int dispatchSCTPDMesg(unsigned char mesg[MAX_UDP_DATA_LENGTH]);


#endif






