/*
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Purpose: interface for a User Adaptation Layer with SCTP daemon program
 * Author : Sim Woon Chiat Siemens ICM N PG U SE D2
 *
 */


#ifndef UA_H
#define UA_H

#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
 #include <config.h>
#endif

#ifndef HAVE_SOCKLEN_T
        typedef int socklen_t;
#endif


#include "sctp.h"

#define MAX_UDP_DATA_LENGTH       1024
/* From sctp.h */
#define SCTP_MAX_IP_LEN           46        /* ==  INET6_ADDRSTRLEN      */
#define SCTP_MAX_NUM_ADDRESSES    20                                
#define SCTP_MAXIMUM_DATA_LENGTH  1400

/* UDP Callbacks data structure to be populated with pointers to the user-defined
   callback functions */
typedef
struct UDP_UA_Callbacks
{

  void (*dataArriveNotif) (unsigned int, unsigned int, 
			   unsigned int, unsigned int, 
			   unsigned int, void*);

  void (*sendFailureNotif) (unsigned int, unsigned char *, 
			    unsigned int, unsigned int *, void*);

  void (*networkStatusChangeNotif) (unsigned int, short, 
				    unsigned short, void*);

  void* (*communicationUpNotif) (unsigned int, unsigned short,
				 int, unsigned short, 
				 unsigned short, void*);

  void (*communicationLostNotif) (unsigned int, unsigned short, void*);

  void (*communicationErrorNotif) (unsigned int, unsigned short, void*);

  void (*restartNotif) (unsigned int, void*);

  void (*shutdownCompleteNotif) (unsigned int, void*);

  void (*queueStatusChangeNotif) (unsigned int, int, 
				  int, int, void*);

  void (*asconfStatusNotif) (unsigned int, unsigned int, 
			     int, void*, void*);

  void (*handle_heartbeat) (unsigned int);

}UDP_UA_CB;


/* Administrative functions */
unsigned short sctpd_initLibrary(int udp_port,int udp_resp_port);

int sctpd_stopLibrary();

int sctpd_eventLoop();

int dispatchUAMesg(unsigned char mesg[MAX_UDP_DATA_LENGTH]);


/* Wrapper functions for sending out SCTP requests to sctpd */
unsigned short sctpd_registerInstance(unsigned short localPort,
                                     unsigned short noOfInStreams,
                                     unsigned short noOfOutStreams,
                                     unsigned int   noOfLocalAddresses,
                                     unsigned char  localAddressList[][SCTP_MAX_IP_LEN],
                                     UDP_UA_CB ULPcallbackFunctions);

int sctpd_unregisterInstance(unsigned short instance_name);

unsigned int sctpd_associate(unsigned short SCTP_InstanceName,
				 unsigned short noOfOutStreams,
				 unsigned char  destinationAddress[],
				 unsigned short destinationPort,
				 void* ulp_data);
int sctpd_shutdown(unsigned int associationID);
int sctpd_abort(unsigned int associationID);
int sctpd_send(unsigned int associationID, 
		   unsigned short streamID,
		   unsigned char *buffer, 
		   unsigned int length, 
		   unsigned int protocolId,
		   short path_id,  
		   void * context, 
		   unsigned int lifetime,
		   int unorderedDelivery,
		   int dontBundle);      				       
unsigned short sctpd_receive(unsigned int   associationID,
			     unsigned short streamID,
			     unsigned char  *buffer,
			     unsigned int   *length,
			     unsigned int flags);
int sctpd_setAssocDefaults(unsigned short SCTP_InstanceName, SCTP_InstanceParameters* params);
int sctpd_getAssocDefaults(unsigned short SCTP_InstanceName, SCTP_InstanceParameters* params);
int sctpd_getAssocStatus(unsigned int associationID, SCTP_AssociationStatus* status);
int sctpd_setAssocStatus(unsigned int associationID, SCTP_AssociationStatus* new_status);
int sctpd_getPathStatus(unsigned int associationID, short path_id, SCTP_PathStatus* status);
int sctpd_setPathStatus(unsigned int associationID, short path_id, SCTP_PathStatus *new_status);
short sctpd_setPrimary(unsigned int associationID, short path_id);
short sctpd_getPrimary(unsigned int associationID);
unsigned int sctpd_getSrttReport(unsigned int associationID, short path_id);
int sctpd_changeHeartBeat(unsigned int associationID,
			      short path_id, int heartbeatON, unsigned int timeIntervall);
int sctpd_requestHeartbeat(unsigned int associationID, short path_id);
int sctpd_setFailureThreshold(unsigned int associationID,
				  unsigned short pathMaxRetransmissions);
int sctpd_deleteAssociation(unsigned int associationID);
int sctpd_getInstanceID(unsigned int associationID, 
			unsigned short* instanceID);

int sctpd_receiveUnsent(unsigned int associationID, unsigned char *buffer, 
			unsigned int *length,unsigned short *streamID, 
			unsigned short *streamSN,unsigned int* protocolId);

int sctpd_receiveUnacked(unsigned int associationID, unsigned char *buffer, 
			 unsigned int *length,unsigned short *streamID, 
			 unsigned short *streamSN,unsigned int* protocolId);

int sctpd_addIPAddress(unsigned int associationID,
		       unsigned char address[SCTP_MAX_IP_LEN],
		       unsigned int *correlationId);

int sctpd_deleteIPAddress(unsigned int associationID, 
			  unsigned char address[SCTP_MAX_IP_LEN],
			  unsigned int* correlationId);

int sctpd_setRemotePrimary(unsigned int associationID, 
			   unsigned char address[SCTP_MAX_IP_LEN]);

int sctpd_getNumberOfLocalAddresses(unsigned int associationID);

unsigned char* sctpd_getLocalAddress(unsigned int associationID, int index);

#endif
